### Name: utilities
### Title: Utility and Miscellaneous Functions
### Aliases: utilities c2s s2c TP.args state.args eos.args GHS
###   can.be.numeric expand.formula protein.length aminoacids
###   thermo.plot.new thermo.postscript axis.label water.lines dPdTtr Ttr
###   ZC label.plot thermo.axis describe basis.comp MP90.cp element
###   mod.obigt which.pmax nonideal change add.protein add.obigt
###   which.balance lsub lsum pprod psum mylapply examples
### Keywords: misc

### ** Examples

  ## Don't show: 
data(thermo)
## End Don't show
  
  ## string to character
  s2c('hello world')
  s2c('hello world',sep=' ',keep.sep=FALSE)
  s2c('3.141592',sep=c('.','9'))
  s2c('3.141592',sep=c('.','9'),move.sep=TRUE)
  # character to string
  c2s(aminoacids())
  c2s(aminoacids(),sep='.')

  ## argument processing
  eos.args('hkf',c('g','H','S','cP','V','kT','e'))
  ## produces an error:
  ## Not run: eos.args('hkf',c('G','H','S','Cp','V','kT','E','Q'))
  thermo$opt$water <- 'IAPWS'  # needed for p and n in next line
  eos.args('water',c('p','u','cv','psat','rho','n','q','x','y','epsilon'))
  TP.args(c(273.15,373.15))
  TP.args(c(273.15,373.15),'Psat')
  TP.args(c(273.15,373.15),c(100,100,200,200))
  state.args(c('AQ','GAS'))
  state.args(c('a','l','liq'))

  ## converting among Gibbs, enthalpy, entropy
  GHS('H') # entropy of H (element)
  # calculate enthalpy of formation of arsenopyrite from the
  # Gibbs energy of formation and entropy                               
  GHS('FeAsS',DG=-33843,S=68.5) 
  # return the value of DG calculated from DH and S
  # cf. -56687.71 from subcrt('water')
  GHS('H2O',DH=-68316.76,S=16.7123)  
 
  ## count specific elements in a formula
  t <- makeup('H2O')
  expand.formula(c('H','O'),t)
  expand.formula(c('C','H','S'),t)

  ## count amino acids in a sequence
  aminoacids('GGSGG')
  aminoacids('WhatAmIMadeOf?')

  ## calculate protein length
  protein.length('LYSC_CHICK')
  # another way to do it
  basis('CHNOS')
  species('LYSC_CHICK')
  protein.length(species()$ispecies)

  ## heat capacity as a function of temperature
  ## (Makhatadze & Privalov, 1990) units: J mol-1
  MP90.cp(c(5,25,50,75,100,125),'LYSC_CHICK')

  ## properties of phase transitions
  t <- info('enstatite')
  # (dP/dT) of transitions
  dPdTtr(t)  # first transition
  dPdTtr(t+1) # second transition
  # temperature of transitions (Ttr) as a function of P
  Ttr(t,P=c(1,10,100,1000))
  Ttr(t,P=c(1,10,100,1000))
  
  ## nominal carbon oxidation states
  ZC('CHNOSZ')
  t <- info(info('LYSC_CHICK'))
  ZC(t$formula)

  ## the stoichiometry of basis species
  basis('CHNOS')
  # in a made-up species
  basis.comp('CHNOS')
  # this one makes a warning because Z isn't
  # in our basis
  basis.comp('CHNOSZ')

  ## describing the basis species
  basis('CHNOSe')
  describe(thermo$basis)
  describe(thermo$basis,T=NULL,P=NULL)

  ## mass and entropy of compounds of elements
  element('CH4')
  element(c('CH4','H2O'),'mass')
  element('Z')   # charge
  # same mass, opposite energy as charge
  element('Z-1') # electron

  ## modify/add species
  info(t <- info('alanine','cr'))
  mod.obigt('alanine',state='cr',G=0,H=0,S=0)
  # now the values of G, H, and S are inconsistent
  # with the elemental composition of alanine
  info(t)
  # add a species
  mod.obigt('myname',formula='CHNOSZ',G=0,H=0)
  info(t <- info('myname'))
  # values of G, H, S as a function of T
  # (without any equations of state parameters)
  subcrt(t)
  



